#ifndef DB_H_INCLUDED
#define DB_H_INCLUDED 1

#include <d32dbms.h>
#include <f32file.h>
#include <s32file.h>
#include "app_context.h"

class CDb : public MContextBase, public CBase {
public:
	IMPORT_C static CDb* NewL(MApp_context& Context, const TDesC& dbname, TInt aFileMode);
	IMPORT_C virtual ~CDb();
	IMPORT_C RDbStoreDatabase& Db();
private:
	CDb(MApp_context& Context);
	void ConstructL(const TDesC& dbname, TInt aFileMode);
	CPermanentFileStore*	iStore;
	RDbStoreDatabase	iDb;
	bool			iDbOpen;
};

class MDBStore {
protected:
	IMPORT_C MDBStore(RDbStoreDatabase& Db);
	IMPORT_C virtual ~MDBStore();
	IMPORT_C void ConstructL(int* columns, int* idx_columns, bool unique_idx,
		const TDesC& name);
	RDbStoreDatabase&	iDb;
	RDbTable		iTable;
	IMPORT_C void SwitchIndexL(TInt Idx); // from 0
	IMPORT_C TInt GetCurrentIndex();
	IMPORT_C void PutL();
	IMPORT_C void SetTextLen(TInt Len);
private:
	int			iCurrentIdx;
	int			iUpdateCount; // for compacting
	int			iTextLen;
	bool			iTableOpen;
};

class CSingleColDbBase  : public MDBStore, public MContextBase, public CBase {
protected:
	CSingleColDbBase(MApp_context& Context, RDbStoreDatabase& Db);
	~CSingleColDbBase();
	bool	SeekL(TInt Idx, bool Updatable=false, bool Add=false);
	void ConstructL(const TDesC& TableName,
		int ColType);
};

template <typename T, typename TMod=T> class CSingleColDb : public CSingleColDbBase {
public:
	static CSingleColDb<T, TMod>* NewL(MApp_context& Context, RDbStoreDatabase& Db, 
		const TDesC& TableName);
	~CSingleColDb();
	bool GetValueL(TInt Idx, TMod& Value);
	void SetValueL(TInt Idx, const T& Value);
	int  ColType() const;
private:
	CSingleColDb(MApp_context& Context, RDbStoreDatabase& Db);
	void ConstructL(const TDesC& TableName);
	void ReadValueL(TMod& Value);
	void WriteValue(const T& Value);
};

#endif
