#ifndef CC_PRESENCE_DATA_H_INCLUDED
#define CC_PRESENCE_DATA_H_INCLUDED 1

#include <e32std.h>
#include "xmlbuf.h"
#include "xml.h"

struct MXmlInfo {
	TTime		iUpdated;
	virtual void	Persist(CXmlBuf* into) = 0;
	MXmlInfo() { iUpdated.HomeTime(); }
	virtual ~MXmlInfo() { }
};

struct TCellInfo : public MXmlInfo {
	TUint		iLAC;
	TUint		iCellId;
	TBuf<20>	iNetwork;
	IMPORT_C virtual void	Persist(CXmlBuf* into);
};

struct TLocInfo : public MXmlInfo {
	HBufC*		iCurrentOrPrevious;
	HBufC*		iNext;
	TTime		iCoPUpdated, iNUpdated;

	TInt		iFlags;
	enum TFlags {
		ECurrent	= 0x0001,
		ENext		= 0x0010,
		EPrevious	= 0x0100
	};
	IMPORT_C virtual void	Persist(CXmlBuf* into);
	
	TLocInfo() : iCurrentOrPrevious(0), iNext(0), iFlags(0) { }
	~TLocInfo() {
		delete iCurrentOrPrevious;
		delete iNext;
	}
};

struct TActivityInfo : public MXmlInfo {
	enum TMode { EUnknown, EActive, EIdle };
	TMode	iMode;
	IMPORT_C virtual void	Persist(CXmlBuf* into);
};

struct TProfileInfo : public MXmlInfo {
	TInt	 iProfileId;
	TInt	 iRingingType;
	TInt	 iRingingVolume;
	TBool	 iVibra;
	TBuf<30> iProfileName;
	IMPORT_C virtual void	Persist(CXmlBuf* into);
};

class CPresenceData : public CBase, public MXmlHandler {
public:
	IMPORT_C static CPresenceData* NewL(const TDesC& Xml,const TTime& send_timestamp);
	IMPORT_C ~CPresenceData();
	IMPORT_C const TCellInfo& CellInfo() const;
	IMPORT_C const TLocInfo& LocInfo() const;
	IMPORT_C const TActivityInfo& ActivityInfo() const;
	IMPORT_C const TProfileInfo& ProfileInfo() const;
	IMPORT_C const TDesC& RawXml() const;
	IMPORT_C const TTime& SendTimeStamp() const;

	IMPORT_C static void MapLocInfo(HBufC * locInfo);
	IMPORT_C static TTime ParseTimeL(const TDesC& Str);
private:
	CPresenceData(const TTime& send_timestamp);
	void ConstructL(const TDesC& Xml);
	void StartElement(const XML_Char *name,
				const XML_Char **atts);

	void EndElement(const XML_Char *name);
	void CharacterData(const XML_Char *s,
				    int len);
	void Error(XML_Error Code, const XML_LChar * String, long ByteIndex);

	void SetStamp(MXmlInfo& Info);
	void ParseStamp(TTime& Stamp);

	enum TParseState {
		EIdle, EEvent, EStamp,
		ECell, ECellLAC, ECellId, ECellNw,
		ELoc, ELocPrevious, ELocCurrent, ELocNext, ELocPoCUpdate,
		EProfile, EProfileId, EProfileName, EProfileRingType, EProfileRingVolume, EProfileVibra,
		EActivity, EActivityMode
	};
	CList<TParseState>* iParseState;

	CXmlParser*	iParser;
	TTime		iStamp;
	HBufC*		iChars;

	TCellInfo	iCellInfo;
	TLocInfo	iLocInfo;
	TActivityInfo	iActivityInfo;
	TProfileInfo	iProfileInfo;
	HBufC*		iXml;
	HBufC*		iError;
	TInt		iErrorCode;

	TTime		iSendTimeStamp;
};

#endif
