#ifndef SYMBIAN_STORE_H_INCLUDED
#define SYMBIAN_STORE_H_INCLUDED 1

#include "store.h"

#include <d32dbms.h>
#include <f32file.h>
#include <s32file.h>

class MDBReclaimableStore : public MReclaimable {
public:
	virtual void SetCurrentStamp(uint16 Stamp);
	virtual void ReclaimL(uint16 upto);
	virtual void ReadFromDbL() { }
	virtual void BeforeDelete() { }
protected:
	MDBReclaimableStore();
	virtual ~MDBReclaimableStore();
	void ConstructL(int* columns, int* idx_columns, bool unique_idx, const TDesC& name, RFs& fs);
	CPermanentFileStore*	iStore;
	RDbStoreDatabase	iDb;
	RDbTable		iTable;
	void PutL();
	uint16			GetStamp() const { return iStamp; }
private:
	int			iUpdateCount; // for compacting
	int			iTimeStampCol; 
	bool			iDbOpen;
	bool			iTableOpen;
	uint16			iStamp;
};

class CCellMap : public CBase, public MCellMap, public MDBReclaimableStore {
public:
	virtual void AddMappingL(uint16 Cell, uint32 Sequence);
	virtual void FindL(uint16 Cell);
	virtual uint32 NextL(); // return 0 when no more matches
	virtual ~CCellMap();
	virtual int Count() const;

	static CCellMap* NewL(RFs& fs);
private:
	CCellMap();
	void ConstructL(RFs& fs);
	bool	iMoreResults;
	uint16	iCurrentCell;

};

class CSequenceMap : public CBase, public MSequenceMap, public MDBReclaimableStore {
public:
	virtual void AddMappingL(uint32 Sequence, uint16 Base);
	virtual void FindL(uint32 Sequence);
	virtual TBaseCount NextL(); // Base==0 wheno no more matches
	virtual ~CSequenceMap();
	virtual int Count() const;
	static CSequenceMap* NewL(RFs& fs);
private:
	CSequenceMap();
	void ConstructL(RFs& fs);
	bool	iMoreResults;
	uint32	iCurrentSequence;
};

class CSequenceIdMap : public CBase, public MSequenceIdMap, public MDBReclaimableStore {
public:
	virtual uint32 AddSequenceL(uint16* Sequence);
	virtual uint32 GetSequenceIdL(uint16* Sequence);
	virtual const uint16* GetSequenceL(uint32 Id);
	virtual ~CSequenceIdMap();
	virtual int Count() const;
	static CSequenceIdMap* NewL(RFs& fs, MGenericIntMap* intmap, CReclaimManager* manager);
	virtual void ReadFromDbL();
	virtual void BeforeDelete();
private:
	CSequenceIdMap();
	void ConstructL(RFs& fs, MGenericIntMap* intmap, CReclaimManager* manager);

	MGenericIntMap*	iRevMap;
	uint32		iNextId;
	CReclaimManager* iManager;

	uint32		iStampCount, iStampGranularity;
	uint16		iLimit;
};


class CMapFactory : public CBase, public MMapFactory {
public:
	virtual MCellMap*	CreateCellMapL();
	virtual MSequenceMap*	CreateSequenceMapL();
	virtual MSequenceIdMap* CreateSequenceIdMapL(int SequenceLen);
	virtual MGenericIntMap*	CreateGenericIntMapL();
	static CMapFactory* NewL(RFs& fs);
private:
	CMapFactory();
	void ConstructL(RFs& fs);
	RFs*	iFs;
	CReclaimManager* iManager;
};


#endif // SYMBIAN_STORE_H_INCLUDED
