#include "settings.h"
#include "symbian_auto_ptr.h"
#include "db.inl"
#include "list.h"
#include <bautils.h>

EXPORT_C CSettings* CSettings::NewL(MApp_context& Context, bool OwnedByContext, const TDesC& SettingName)
{
	auto_ptr<CSettings> ret(new (ELeave) CSettings(Context, OwnedByContext));
	ret->ConstructL(SettingName);
	return ret.release();
}

CSettings::~CSettings()
{
	delete iListeners;
	delete iIntStore;
	delete iTimeStore;
	delete iDesStore;
	delete iDb;
}

bool CSettings::GetSettingL(TInt Setting, TDes& Value)
{
	return iDesStore->GetValueL(Setting, Value);
}

bool CSettings::GetSettingL(TInt Setting, TInt& Value)
{
	return iIntStore->GetValueL(Setting, Value);
}

bool CSettings::GetSettingL(TInt Setting, TTime& Value)
{
	return iTimeStore->GetValueL(Setting, Value);
}

void CSettings::WriteSettingL(TInt Setting, const TDesC& Value)
{
	iDesStore->SetValueL(Setting, Value);
	NotifyOfChange(Setting);
}

void CSettings::WriteSettingL(TInt Setting, const TInt& Value)
{
	iIntStore->SetValueL(Setting, Value);
	NotifyOfChange(Setting);
}

void CSettings::WriteSettingL(TInt Setting, const TTime& Value)
{
	iTimeStore->SetValueL(Setting, Value);
	NotifyOfChange(Setting);
}

CSettings::CSettings(MApp_context& Context, bool OwnedByContext) : MContextBase(Context, OwnedByContext)
{
}

void ListDeletor(void* p)
{
	CList<MSettingListener*>* l=(CList<MSettingListener*>*)p;
	delete l;
}

void CSettings::ConstructL(const TDesC& SettingName)
{
	_LIT(db_base, "SETTINGS");
	TBuf<100> db;
	db.Append(db_base);
	db.Append(SettingName);
	TFileName oldfilen, newfilen;
	oldfilen.Format(_L("%S%S.db"), &AppDir(), &db_base);
	newfilen.Format(_L("%S%S.db"), &AppDir(), &db);
	if (BaflUtils::FileExists(Fs(), oldfilen)) {
		User::LeaveIfError(Fs().Rename(oldfilen, newfilen));
	}

	iListeners=CGenericIntMap::NewL();
	iListeners->SetDeletor(ListDeletor);

	iDb=CDb::NewL(AppContext(), db, EFileRead|EFileWrite );
	if (OwnedByContext()) iDb->SetOwnedByContext();
	iDesStore=CSingleColDb<TDesC, TDes>::NewL(AppContext(), iDb->Db(), _L("DES"));
	if (OwnedByContext()) iDesStore->SetOwnedByContext();
	iIntStore=CSingleColDb<TInt>::NewL(AppContext(), iDb->Db(), _L("INT"));
	if (OwnedByContext()) iIntStore->SetOwnedByContext();
	iTimeStore=CSingleColDb<TTime>::NewL(AppContext(), iDb->Db(), _L("TIME"));
	if (OwnedByContext()) iTimeStore->SetOwnedByContext();
}

void CSettings::NotifyOnChange(TInt Setting, MSettingListener* Listener)
{
	CList<MSettingListener*>* l=(CList<MSettingListener*>*)iListeners->GetData(Setting);
	if (!l) {
		l=CList<MSettingListener*>::NewL();
		iListeners->AddDataL(Setting, (void*)l);
	}
	l->AppendL(Listener);
}

void CSettings::CancelNotifyOnChange(TInt Setting, MSettingListener* Listener)
{
	CList<MSettingListener*>* l=(CList<MSettingListener*>*)iListeners->GetData(Setting);
	if (!l) {
		return;
	}
	CList<MSettingListener*>::Node *i, *prev, *tmp;
	i=l->iFirst; prev=0;
	while (i) {
		if (i->Item==Listener) {
			if (prev) {
				prev->Next=i->Next;
			} else {
				l->iFirst=i->Next;
			}
			if (i==l->iCurrent) l->iCurrent=prev;
			tmp=i;
			i=i->Next;
			delete tmp;
		} else {
			prev=i;
			i=i->Next;
		}
	}
}

void CSettings::NotifyOfChange(TInt Setting)
{
	CList<MSettingListener*>* l=(CList<MSettingListener*>*)iListeners->GetData(Setting);
	if (!l) {
		return;
	}
	CList<MSettingListener*>::Node *i;
	i=l->iFirst;
	while (i) {
		i->Item->SettingChanged(Setting);
		i=i->Next;
	}
}
