#include "symbian_tree.h"
#include "pointer.h"

EXPORT_C void CGenericIntMap::AddDataL(uint32 Key, void* data, bool overwrite)
{
	if (overwrite) {
		DeleteL(Key);
		iCount--;
	}
	TBtreePos pos;
	Entry e;
	e.key=Key; e.data=data;
	TBool no_dup=iTree->InsertL(pos, e);
	if (no_dup) iCount++;
	if (! no_dup && iDeletor) {
		(*iDeletor)(data);
	}
}

EXPORT_C void* CGenericIntMap::GetData(uint32 Key)
{
	TBtreePos pos;
	TBool found=iTree->FindL(pos, Key);
	if (found) {
		Entry e;
		iTree->ExtractAtL(pos, e);
		return e.data;
	} else {
		return 0;
	}
}

EXPORT_C void CGenericIntMap::DeleteL(uint32 Key)
{
	TBtreePos pos;
	TBool found=iTree->FindL(pos, Key);
	if (found) {
		if (iDeletor) {
			Entry e;
			iTree->ExtractAtL(pos, e);
			(*iDeletor)(e.data);
		}
		iTree->DeleteAtL(pos);
		iCount--;
	}
}

EXPORT_C CGenericIntMap::~CGenericIntMap()
{
	if (iDeletor) {
		TBtreeMark i;
		TBool more=iTree->ResetL(i);
		Entry e;
		while (more) {
			iTree->ExtractAtL(i, e);
			(*iDeletor)(e.data);
			more=iTree->NextL(i);
		}
	}

	delete iTree;
	delete iPool;
}

EXPORT_C void CGenericIntMap::Reset()
{
	if (iDeletor) {
		TBtreeMark i;
		TBool more=iTree->ResetL(i);
		Entry e;
		while (more) {
			iTree->ExtractAtL(i, e);
			(*iDeletor)(e.data);
			more=iTree->NextL(i);
		}
	}
	iTree->ClearL();
	iCount=0;
}

EXPORT_C int CGenericIntMap::Count() const
{
	return iCount;
}

EXPORT_C void CGenericIntMap::SetDeletor( void(*delete_func)(void* data) )
{
	iDeletor=delete_func;
}

EXPORT_C CGenericIntMap* CGenericIntMap::NewL()
{
	auto_ptr<CGenericIntMap> ret(new (ELeave) CGenericIntMap);
	ret->ConstructL();
	return ret.release();
}

CGenericIntMap::CGenericIntMap()
{
}

void CGenericIntMap::ConstructL()
{
	iTree=new (ELeave) TBtreeFix<Entry, uint32>(EBtreeFast);
	iPool=CMemPagePool::NewL();
	iTree->Connect(iPool, &iKey);
}

void CGenericIntMap::GenKey::Between(const TAny* aLeft,const TAny* aRight,TBtreePivot& aPivot) const
{
	uint32 left=*(uint32*)aLeft;
	uint32 right=*(uint32*)aRight;

	uint32 mid=left+(right-left)/2;
	aPivot.Copy((TUint8*)&mid, sizeof(mid));
}


TInt CGenericIntMap::GenKey::Compare(const TAny* aLeft,const TAny* aRight) const
{
	uint32 left=*(uint32*)aLeft;
	uint32 right=*(uint32*)aRight;

	if (left<right) return -1;
	if (left>right) return 1;
	return 0;
}

const TAny* CGenericIntMap::GenKey::Key(const TAny* anEntry) const
{
	Entry* e=(Entry*)anEntry;
	return (TAny*)&(e->key);
}