#ifndef CL_PRESENCE_PUBLISHER_H_INCLUDED
#define CL_PRESENCE_PUBLISHER_H_INCLUDED 1

#include "app_context.h"
#include "ContextClientSession.h"
#include "ftp.h"
#include "xmlbuf.h"
#include "presence_data.h"
#include "status_notif.h"

class MPresencePublisherListener
{
public:
	virtual void NotifyNewPresence(const TDesC& Xml)=0;
};

class CPresencePublisher : public CCheckedActive, public Mlogger, public MContextBase, 
		public MTimeOut, public MSettingListener {
public:
	static CPresencePublisher* NewL(MApp_context& Context, i_status_notif* CallBack, MPresencePublisherListener& aListener );
	~CPresencePublisher();

	void SuspendConnection();
	void ResumeConnection();
	bool ConnectionSuspended();
	bool Disabled();
	

private:
	CPresencePublisher(MApp_context& Context, i_status_notif* CallBack, MPresencePublisherListener& aListener );
	void ConstructL();
	void Restart();
	void SendUpdate();
	void MakePresence();

	// Mlogger
	virtual void register_source(const TDesC& name, const TDesC& initial_value, const TTime& time);
	virtual void new_value(log_priority priority, const TDesC& name, const TDesC& value, const TTime& time);
	virtual void unregister_source(const TDesC& name, const TTime& time);
	virtual const TDesC& name() const;

	// CCheckedActive
	void CheckedRunL();
	void DoCancel();
	virtual TInt CheckedRunError(TInt aError);

	// MTimeOut
	void expired();

	// MSettingListener
	void SettingChanged(TInt Setting);

	// data
	enum EState { EUnknown, EReconnecting, EConnecting, EConnected, 
		ESendingUpdate, ESuspending, ESuspended, EResuming, EDisabling, EDisabled };
	EState			iCurrentState;
	enum ENextOp { ENone, ESuspend, EResume, EDisable, EEnable };
	ENextOp			iNextOp;

	bool			iNewValue;

	RContextClientSession	iSession;
	TBuf<30>		iUser, iPass, iServer;
	CTimeOut*		iWait;
	TInt			iWaitTime;
	TInt			iAP;
	CXmlBuf*		iPresence;
	HBufC*			iSendPresence;

	TCellInfo		iCell;
	TLocInfo		iLocation;
	TActivityInfo		iActivity;
	TProfileInfo		iProfile;

	i_status_notif*		iCallBack;

	MPresencePublisherListener& iListener;

};




#endif
