#include "circular.h"
#include "symbian_auto_ptr.h"

#include "app_context.h"

CCircularLog* CCircularLog::NewL(int Size, bool Reverse)
{
	CALLSTACKITEM(_L("CCircularLog::NewL"));

	auto_ptr<CCircularLog> ret(new (ELeave) CCircularLog(Reverse));
	ret->ConstructL(Size);
	return ret.release();
}

void CCircularLog::AddL(const TDesC& String)
{
	CALLSTACKITEM(_L("CCircularLog::AddL"));

	if (iCount<iSize) {
		iCount++;
		iArray->AppendL(String);
	} else {
		iArray->Delete(iPos-1);
		iArray->InsertL(iPos-1, String);
		iPos++;
		if (iPos>iSize) iPos=1;
	}
	if (iObserver) iObserver->ContentsChanged();
}

CCircularLog::~CCircularLog()
{
	CALLSTACKITEM(_L("CCircularLog::~CCircularLog"));

	delete iArray;
}

CCircularLog::CCircularLog(bool Reverse)
{
	CALLSTACKITEM(_L("CCircularLog::CCircularLog"));

	iReverse=Reverse;
}

void CCircularLog::ConstructL(int Size)
{
	CALLSTACKITEM(_L("CCircularLog::ConstructL"));

	iArray=new (ELeave) CDesCArrayFlat(Size);
	iSize=Size;
	iPos=1;
}

TInt CCircularLog::MdcaCount() const
{
	CALLSTACKITEM(_L("CCircularLog::MdcaCount"));

	return iCount;
}

TPtrC16 CCircularLog::MdcaPoint(TInt aIndex) const
{
	CALLSTACKITEM(_L("CCircularLog::MdcaPoint"));

	int idx;
	if (!iReverse) {
		idx=aIndex+iPos;
		if (idx>=iCount) idx-=iCount+1;
	} else {
		if (iPos==1) {
			idx=iCount-aIndex-1;
		} else {
			idx=iPos-aIndex-2;
			if (idx<0) idx+=iCount;
		}
	}
	return iArray->MdcaPoint(idx);
}

void CCircularLog::SetObserver(MListObserver* Observer)
{
	CALLSTACKITEM(_L("CCircularLog::SetObserver"));

	iObserver=Observer;
}