#include "locationing.h"
#include <PLPVAR.H>
#include <PLPVARIANT.H>

#ifndef __WINS__
#define WAIT_TIME	90
#else
#define WAIT_TIME	1
#endif

void Clocationing::ConstructL(i_status_notif* i_cb, MNaming* i_naming, sms* i_sms_handler)
{
	CALLSTACKITEM(_L("Clocationing::ConstructL"));

	iName=(TText*)L"Clocationing";	
	iNaming=i_naming;
	cb=i_cb;
	
	smsh = i_sms_handler;

	iTimer=CTimeOut::NewL(*this);

#ifndef __WINS__
	MAdvGsmPhoneInformation::TSubscriberId aImsi;
	if (Phone().GetSubscriberId(aImsi)!=KErrNone) {
		TPlpVariantMachineId machineId;
		PlpVariant::GetMachineIdL(machineId);
		imsi=machineId;
	} else {
		imsi.Copy(aImsi);
	}
#else
	// Return a fake IMEI when working on emulator
	_LIT(KEmulatorImsi, "244050000000000");
	imsi.Copy(KEmulatorImsi);
#endif
	smsh->AddHandler(this);

}

Clocationing::Clocationing(MApp_context& Context) : MContextBase(Context)
{
	CALLSTACKITEM(_L("Clocationing::Clocationing"));

}

Clocationing* Clocationing::NewL(MApp_context& Context, i_status_notif* i_cb, MNaming* i_naming, sms * i_sms_handler)
{
	CALLSTACKITEM(_L("Clocationing::NewL"));

	auto_ptr<Clocationing> ret(new (ELeave) Clocationing(Context));

	ret->ConstructL(i_cb, i_naming, i_sms_handler);
	return ret.release();
}

void Clocationing::handle_error(const TDesC& descr)
{
	CALLSTACKITEM(_L("Clocationing::handle_error"));

	cb->error(descr);
}

void Clocationing::handle_move(const TMsvId& msg_id, const TMsvId& from_folder, const TMsvId& to_folder, const TDesC& sender)
{
}

void Clocationing::handle_delete(const TMsvId& msg_id, const TMsvId& parent_folder, const TDesC& sender)
{
}

void Clocationing::handle_sending(const TMsvId& entry_id, const TDesC& sender, const TDesC & body)
{
}
void Clocationing::handle_change(const TMsvId& msg_id, const TDesC& sender)
{
}

void Clocationing::handle_read(const TMsvId& msg_id, const TDesC& sender)
{
}


bool Clocationing::handle_reception(const TMsvId& entry_id, const TMsvId& folder_id, const TDesC& sender, const TDesC& body)
{
	CALLSTACKITEM(_L("Clocationing::handle_reception"));

	if (sender.Compare(_L("16507")) ) {
		//cb->status_change(sender);
		return false;
	}
	if (iToDiscard) {
		--iToDiscard;
		cb->status_change(_L("discarded unreliable result"));
		return true;
	}
	if (!iToName.SentRequest) return false;

#ifndef __WINS__
	/*
	 * FIXME: why was this done?
	 *
	TTime comp;
	comp.HomeTime();
	comp-=TTimeIntervalSeconds(WAIT_TIME);
	if (iToName.Time<comp) {
		iToName.Reset();
		cb->error(_L("Result too old"));
		return true;
	}
	*/
#endif
	
	if (body.Left(8).Compare(_L("itse  on"))) {
		if (true || iRetryCount>5) {
			/*
			 * It seems that if the locationing doesn't
			 * work, it won't work again before a cell change.
			 * The Radiolinja service probably sometimes doesn't
			 * get updated correctly, and doesn't retry.
			 * It's not worth trying again before we go
			 * to a new cell.
			 */
			iToName.Reset();
			cb->error(_L("Error result"));
		} else {
			iToName.SentRequest=false;
			iRetryCount++;
			iTimer->Wait(WAIT_TIME*4);
		}
		return true;
	}

	HBufC* content=body.Mid(19, body.Length()-19-1).AllocLC();
	content->Des().Trim();

	cb->status_change(*content);

	iNaming->add_cellid_name(iToName.CellId, *content);
	CleanupStack::PopAndDestroy(); //content
	iToName.Reset();

	return true;
}

void Clocationing::GetNameL(const TDesC& CellId)
{
	CALLSTACKITEM(_L("Clocationing::GetNameL"));

	if (!iLocationingAvailable) return;

	if (iToName.SentRequest) {
		return;
	}
	iToName.CellId=CellId;
	iToName.SentRequest=false;
	iTimer->Wait(WAIT_TIME);
	iRetryCount=0;
}

void Clocationing::expired()
{
	CALLSTACKITEM(_L("Clocationing::expired"));

	iToName.Time.HomeTime();
	TInt err=smsh->send_message(_L("16507"), _L("PAIKKA itse"), false);
	if (err!=KErrNone) {
		TBuf<20> err;
		err.Format(_L("sms error %d"), err);
		cb->error(err);
		return;
	}
	iToName.SentRequest=true;
}

void Clocationing::test()
{
	CALLSTACKITEM(_L("Clocationing::test"));

	if (!iLocationingAvailable) {
		cb->error(_L("Locationing not available"));
		return;
	}

	TInt err=smsh->send_message(_L("16507"), _L("PAIKKA itse"), false);
	if (err!=KErrNone) {
		TBuf<20> err;
		err.Format(_L("sms error %d"), err);
		cb->error(err);
	}
}

Clocationing::~Clocationing()
{
	CALLSTACKITEM(_L("Clocationing::~Clocationing"));

	delete iTimer;
}

void Clocationing::now_at_location(const TDesC& cell, TInt id, bool is_base, bool loc_changed, TTime time)
{
	CALLSTACKITEM(_L("Clocationing::now_at_location"));

	if ( cell.Find(_L("RADIOLINJA"))==KErrNotFound) {
		iLocationingAvailable=false;
	} else {
		iLocationingAvailable=true;
	}

	if (!iLocationingAvailable) return;

	if (loc_changed) {
		iTimer->Reset();
		if (iToName.SentRequest) {
			iToName.SentRequest=false;
			++iToDiscard;
		}
	}
}
