#include "log_profile.h"
#include "symbian_auto_ptr.h"
#include <e32std.h>
#define INTERVAL 30

Clog_profile::Clog_profile(MApp_context& Context) : CCheckedActive(CCheckedActive::EPriorityIdle, _L("Clog_profile")), Mlog_base_impl(Context)
{
	CALLSTACKITEM(_L("Clog_profile::Clog_profile"));

}

void Clog_profile::ConstructL()
{
	CALLSTACKITEM(_L("Clog_profile::ConstructL"));

	profile_value=HBufC::NewL(40);
	name=HBufC::NewL(50);
	Mlog_base_impl::ConstructL(KLog_profile, _L("%d %S (%d %d %S)"));

	aConditions=new (ELeave) CArrayFixFlat<TSysAgentCondition>(sizeof(TSysAgentCondition));
	aConditions->SetReserveL(1);
	
	CActiveScheduler::Add(this);
	listen_for_notification();
}

void Clog_profile::listen_for_notification()
{
	CALLSTACKITEM(_L("Clog_profile::listen_for_notification"));

	Cancel();

	prev_profile=get_profile_via_profilapi();
	TSysAgentCondition cond(KUidProfile, prev_profile, ESysAgentNotEquals);
	if (aConditions->Count()==0) {
		aConditions->AppendL(cond);
	} else {
		aConditions->At(0)=cond;
	}

	TInt ret=SysAgent().NotifyOnCondition(*aConditions, iStatus);
	if (ret==KErrNone) {
		SetActive();
	} else {
		iStatus=ret;
		iWait->Wait(5);
	}
}

IMPORT_C void Clog_profile::expired(CBase* Source)
{
	CALLSTACKITEM(_L("Clog_profile::expired"));

	CheckedRunL();
}

Clog_profile* Clog_profile::NewL(MApp_context& Context)
{
	CALLSTACKITEM(_L("Clog_profile::NewL"));

	Clog_profile* ret;
	ret=new (ELeave) Clog_profile(Context);
	CleanupStack::PushL(ret);
	ret->ConstructL();
	CleanupStack::Pop();
	return ret;
}

void Clog_profile::DoCancel()
{
	CALLSTACKITEM(_L("Clog_profile::DoCancel"));

	SysAgent().NotifyEventCancel();
}

Clog_profile::~Clog_profile()
{
	CALLSTACKITEM(_L("Clog_profile::~Clog_profile"));

	delete iWait;
	Cancel();

	delete name;
	delete profile_value;
	delete aConditions;

}

TInt Clog_profile::CheckedRunError(TInt /*aError*/)
{
	CALLSTACKITEM(_L("Clog_profile::CheckedRunError"));

	Cancel();
	listen_for_notification();
	return KErrNone;
}

void Clog_profile::CheckedRunL()
{
	CALLSTACKITEM(_L("Clog_profile::CheckedRunL"));

	if (iStatus!=KErrNone) {
		SysAgent().Close();
		SysAgent().Connect();
	}

	iProfileUid=get_profile_via_profilapi();

	if (prev_profile==iProfileUid) {
		listen_for_notification();
		return;
	}

	prev_profile=iProfileUid;

	TPtrC namep=name->Des();
	profile_value->Des().Format(*log_format, iProfileUid, &(namep), iRingingType, iRingingVolume, &iVibraDescr);
	post_new_value(profile_value->Des());

	listen_for_notification();
}

const TDesC& Clog_profile::get_value()
{
	CALLSTACKITEM(_L("Clog_profile::get_value"));

	TPtrC namep=name->Des();
	profile_value->Des().Format(*log_format, prev_profile, &(namep), iRingingType, iRingingVolume,
		&iVibraDescr);
	return *profile_value;
}

TInt Clog_profile::get_profile_via_profilapi()
{
	CALLSTACKITEM(_L("Clog_profile::get_profile_via_profilapi"));

	TInt profileUid=0;
#ifndef __WINS__

	auto_ptr<CProfileAPI> profile(CProfileAPI::NewL());
	profile->GetProfileActiveName(name->Des(), &profileUid); // Get active profile

	TFileName dummy1, dummy2;
	TInt dummy3, dummy4;
	auto_ptr<CArrayFixFlat<TContactItemId> > dummy6(new (ELeave) CArrayFixFlat<TContactItemId>(1));
	profile->GetProfileMultiData(dummy1, dummy2, iRingingType, iRingingVolume, iVibra, dummy3, dummy6.get(),
		dummy4);
#else
	iVibra=true;
	iRingingType=3;
	iRingingVolume=2;
	*name=_L("Genrl");

#endif
	if (iVibra) {
		iVibraDescr=_L("On");
	} else {
		iVibraDescr=_L("Off");
	}
	return profileUid;
}
