#include "transfer.h"
#include "context_log.rsg"
#include "symbian_auto_ptr.h"
#include <plpvariant.h>
#include "cl_settings.h"

_LIT(cellid_filen, "c:\\system\\apps\\context_log\\cellid_names.txt");
_LIT(transfer_cellid_filen, "c:\\system\\apps\\context_log\\cellid_names_trans.txt");

CTransferBase::CTransferBase(MApp_context& Context) : MContextBase(Context)
{
	CALLSTACKITEM(_L("CTransferBase::CTransferBase"));

}

CSendUITransfer::CSendUITransfer(MApp_context& Context) : CTransferBase(Context)
{
	CALLSTACKITEM(_L("CSendUITransfer::CSendUITransfer"));

}


void CTransferBase::ConstructL(i_status_notif* callback, const TDesC& dir1, const TDesC& dir2)
{
	CALLSTACKITEM(_L("CTransferBase::ConstructL"));

	send_dirs=new CDesC16ArrayFlat(5);
	send_leave=new CArrayFixFlat<bool>(5);
	dir_prefix1=dir1;
	if (dir1.Length() && dir1.Right(1).Compare(_L("\\")) ) { dir_prefix1.Append(_L("\\")); }
	dir_prefix2=dir2;
	if (dir2.Length() && dir2.Right(1).Compare(_L("\\")) ) { dir_prefix2.Append(_L("\\")); }
	cb=callback;
}

void CSendUITransfer::ConstructL(i_status_notif* callback, TInt cmdid, const TDesC& dir1, const TDesC& dir2)
{
	CALLSTACKITEM(_L("CSendUITransfer::ConstructL"));

	CTransferBase::ConstructL(callback, dir1, dir2);
	sendui=CSendAppUi::NewL(cmdid, NULL);
}

CSendUITransfer* CSendUITransfer::NewL(MApp_context& Context, i_status_notif* callback, TInt cmdid, 
				       const TDesC& dir1, const TDesC& dir2)
{
	CALLSTACKITEM(_L("CSendUITransfer::NewL"));

	auto_ptr<CSendUITransfer> ret(new (ELeave) CSendUITransfer(Context));
	ret->ConstructL(callback, cmdid, dir1, dir2);
	return ret.release();
}

CTransferBase::~CTransferBase()
{
	CALLSTACKITEM(_L("CTransferBase::~CTransferBase"));

	delete send_dirs;
	delete send_leave;
	delete file_names;
}

CSendUITransfer::~CSendUITransfer()
{
	CALLSTACKITEM(_L("CSendUITransfer::NewL"));

	delete sendui;
}

void CSendUITransfer::DisplayMenuL(CEikMenuPane& aMenuPane)
{
	CALLSTACKITEM(_L("CSendUITransfer::DisplayMenuL"));

	sendui->DisplaySendCascadeMenuL(aMenuPane);
}

void CSendUITransfer::DisplaySendMenuL(CEikMenuPane& aMenuPane, TInt pos)
{
	CALLSTACKITEM(_L("CSendUITransfer::DisplaySendMenuL"));

	sendui->DisplaySendMenuItemL(aMenuPane, pos, 
		TSendingCapabilities (0, 100000, 0));
}

void CTransferBase::add_filesL(const TDesC& file, bool leave_last)
{
	CALLSTACKITEM(_L("CTransferBase::add_filesL"));

	TFileName filen;
	filen=dir_prefix1;
	filen.Append(file);
	send_dirs->AppendL(filen);
	send_leave->AppendL(leave_last);

	if (dir_prefix2.Length()) {
		filen=dir_prefix2;
		filen.Append(file);
		send_dirs->AppendL(filen);
		send_leave->AppendL(false);
	}
}

bool CTransferBase::GetFiles(TTime AfterTime)
{
	CALLSTACKITEM(_L("CTransferBase::GetFiles"));

	if (send_dirs->Count()==0) return true;

	delete file_names;
	file_names=new CDesC16ArrayFlat(10);
	TEntry fe;
	TFileName filen;
	TFileName dirname;

	int i;
	for (i=0; i<send_dirs->Count(); i++) {
		dirname=(*send_dirs)[i];
		bool leave=(*send_leave)[i];
		CDir* dir=0; 

		User::LeaveIfError(Fs().GetDir(dirname, KEntryAttNormal, ESortByName, dir));
		CleanupStack::PushL(dir);

		int j;
		int count=dir->Count();
		if (leave) --count;
		TParse p;
		p.Set(dirname, 0, 0);
		for (j=0; j<count; j++) {
			filen=p.DriveAndPath();
			filen.Append((*dir)[j].iName);
			if (Fs().Entry(filen, fe)==KErrNone) {
				if (fe.iSize>0 && fe.iModified>AfterTime) {
					file_names->AppendL(filen);
				} else {
					Fs().Delete(filen);
				}
			}
		}
		CleanupStack::PopAndDestroy(dir);
	}
	if (file_names->Count()>0) return false;
	return true;
}

bool CSendUITransfer::transfer_files(TInt cmdid)
{
	CALLSTACKITEM(_L("CSendUITransfer::transfer_files"));

	if (in_progress) return false;

	in_progress=true;

	if (GetFiles()) {
		in_progress=false;
		return true;
	}

	sendui->CreateAndSendMessageL (cmdid, 0, file_names);

	CAknQueryDialog* dlg = new(ELeave) CAknQueryDialog(CAknQueryDialog::ENoTone);
	CleanupStack::PushL(dlg);
	
	_LIT(pr, "Delete files?");
	dlg->SetPromptL(pr);

	CleanupStack::Pop(); //dlg

	if (dlg->ExecuteLD(R_CL_CONFIRMATION_QUERY_DIALOG)) {
		if (cb) cb->status_change(_L("deleting files"));
		for (int i=0; i<file_names->Count(); i++) {
			Fs().Delete((*file_names)[i]);
		}
		if (cb) cb->status_change(_L("deleted files"));
	}

	in_progress=false;

	return true;
}

CFtpTransfer* CFtpTransfer::NewL(MApp_context& Context, 
	i_status_notif* callback, const TDesC& dir1, const TDesC& dir2,
		TConnType ConnType)
{
	CALLSTACKITEM(_L("CFtpTransfer::NewL"));

	auto_ptr<CFtpTransfer> ret(new (ELeave) CFtpTransfer(Context));
	ret->ConstructL(callback, dir1, dir2, ConnType);
	return ret.release();
}

void CFtpTransfer::ConstructL(i_status_notif* callback, const TDesC& dir1, const TDesC& dir2,
		TConnType ConnType)
{
	CALLSTACKITEM(_L("CFtpTransfer::ConstructL"));

	CTransferBase::ConstructL(callback, dir1, dir2);
	iConnType=ConnType;
}

CFtpTransfer::CFtpTransfer(MApp_context& Context) : CTransferBase(Context)
{
	CALLSTACKITEM(_L("CFtpTransfer::CFtpTransfer"));

	current_state=IDLE;
}

void CFtpTransfer::TransferFiles(const TDesC& Meta, TTime AfterTime)
{
	CALLSTACKITEM(_L("CFtpTransfer::TransferFiles"));

	if (current_state!=IDLE) return;

	iMeta=Meta;

	if (GetFiles(AfterTime)) {
		cb->finished();
		return;
	}

	file_index=0;
	delete http; http=0;
	http=CHttp::NewL(AppContext(), *this);
	current_state=CONNECTING;
	TInt Iap;
	Settings().GetSettingL(SETTING_IP_AP, Iap);
	TBuf<100> url;
	if (! Settings().GetSettingL(SETTING_UPLOAD_URLBASE, url)) {
		url=DEFAULT_UPLOAD_URLBASE;
	}
	url.Append(_L("put3.pl"));
	http->Connect(Iap, url);
}

CFtpTransfer::~CFtpTransfer()
{
	CALLSTACKITEM(_L("CHttp::NewL"));

	delete http;
}


void CFtpTransfer::success(CBase* /*source*/)
{
	CALLSTACKITEM(_L("CFtpTransfer::success"));

	TBuf<40> msg;
again:
	switch(current_state) {
	case IDLE:
		cb->error(_L("inconsistent state"));
		break;
	case CWDING:
		msg.Format(_L("Sending %d"), file_index);
		cb->status_change(msg);
		current_state=SENDING_FILE;
		http->Store((*file_names)[file_index], iMeta);
		break;
	case SENDING_FILE:
		Fs().Delete((*file_names)[file_index]);
		++file_index;
		if (file_index<file_names->Count()) {
			msg.Format(_L("Sending %d"), file_index);
			cb->status_change(msg);
			http->Store((*file_names)[file_index], iMeta);
		} else {
			cb->status_change(_L("closing"));
			current_state=CLOSING;
			if (http) http->Close();
		}
		break;
	case CLOSING:
		delete http; http=0;
		current_state=IDLE;
		cb->finished();
		break;
	case CONNECTING:
		if (GetImsi(iImsi)) {
			http->Cwd(iImsi);
			current_state=CWDING;
			goto again;
		} else {
			current_state=CLOSING;
			if (http) http->Close();
			cb->error(_L("Cannot get IMSI"));
		}
		break;
	}
}

bool CFtpTransfer::GetImsi(TDes& aImsi)
{
	CALLSTACKITEM(_L("CFtpTransfer::GetImsi"));

#ifndef __WINS__1
	TPlpVariantMachineId machineId;
	PlpVariant::GetMachineIdL(machineId);
	aImsi=machineId;
#else
	// Return a fake IMEI when working on emulator
	_LIT(KEmulatorImsi, "244050000000000");
	aImsi.Copy(KEmulatorImsi);
#endif
	return true;
}

void CFtpTransfer::error(CBase* source, TInt /*code*/, const TDesC& reason)
{
	CALLSTACKITEM(_L("CFtpTransfer::error"));

	if (current_state==MKDING) {
		// probably exists already, let's try CWDing
		success(source);
	} else {
		cb->error(reason);
		delete http; http=0;
		current_state=IDLE;
	}
}

void CFtpTransfer::info(CBase* /*source*/, const TDesC& msg)
{
	CALLSTACKITEM(_L("CFtpTransfer::info"));

	cb->status_change(msg);
}

CPeriodicTransfer* CPeriodicTransfer::NewL(MApp_context& Context, int hours, MSocketObserver* callback)
{
	CALLSTACKITEM(_L("CPeriodicTransfer::NewL"));

	auto_ptr<CPeriodicTransfer> ret(new (ELeave) 
		CPeriodicTransfer(Context, hours, callback));
	ret->ConstructL();
	return ret.release();
}

CPeriodicTransfer::~CPeriodicTransfer()
{
	CALLSTACKITEM(_L("CPeriodicTransfer::NewL"));

	delete iTimer;
	delete iFtp;
	delete iCommLog;
}

CPeriodicTransfer::CPeriodicTransfer(MApp_context& Context, int hours, MSocketObserver* callback) : 
MContextBase(Context), iHours(hours), iCb(callback)
{
	CALLSTACKITEM(_L("CPeriodicTransfer::CPeriodicTransfer"));

}


void CPeriodicTransfer::ConstructL()
{
	CALLSTACKITEM(_L("CPeriodicTransfer::ConstructL"));

	iTimer=CTimeOut::NewL(*this);
	iFtp=CFtpTransfer::NewL(AppContext(), this, DataDir(), AppDir(), CFtpTransfer::HTTP);
	iFtp->add_filesL(_L("log*txt"), true);
	iFtp->add_filesL(_L("book*txt"), true);
	iFtp->add_filesL(_L("comm*txt"), false);
	iFtp->add_filesL(_L("rec*amr"), false);
	iFtp->add_filesL(_L("cellid_names.txt"), false);
	iCommLog=Clog_comm::NewL(AppContext(), this);
	current_state=IDLE;
	iTimer->Wait(60*60);
}

void CPeriodicTransfer::expired()
{
	CALLSTACKITEM(_L("CPeriodicTransfer::expired"));

	TTime now;
	now.HomeTime();

	if (++iCount>=iHours || (iHours==24 && now.DateTime().Hour()==3) ) {
			Transfer();
	} else {
		iTimer->Wait(60*60);
        //iTimer->Wait(1);
	}
}

void CPeriodicTransfer::Transfer(bool MakeCommLog)
{
	CALLSTACKITEM(_L("CPeriodicTransfer::Transfer"));

	iTimer->Reset();
	iCb->info(this, _L("Getting comm log"));
	current_state=GETTING_COMM_LOG;
	if (MakeCommLog) {
		iCommLogFrom=0;
		TTime existing(0);
		TRAPD(err, Settings().GetSettingL(SETTING_LAST_COMMLOG_UPLOAD, iCommLogFrom));
		iCommLog->LogsExist(existing);
		if (existing>iCommLogFrom) iCommLogFrom=existing;
		iCommLog->write_comm_log(iCommLogFrom);
	} else {
		finished();
	}
}

void CPeriodicTransfer::finished()
{
	CALLSTACKITEM(_L("CPeriodicTransfer::finished"));

	if (current_state==GETTING_COMM_LOG) {
		iCb->info(this, _L("FTPing"));
		current_state=FTPING;
		iFtp->TransferFiles();
	} else {
		iCount=0;
		current_state=IDLE;
		TRAPD(err, Settings().WriteSettingL(SETTING_LAST_COMMLOG_UPLOAD, iCommLogFrom));
		iTimer->Wait(60*60);
		iCb->info(this, _L("Transferred"));
	}
}

void CPeriodicTransfer::error(const TDesC& reason)
{
	CALLSTACKITEM(_L("CPeriodicTransfer::error"));

	current_state=IDLE;
	iTimer->Wait(60*60);
	iCb->error(this, 1, reason);
}

void CPeriodicTransfer::status_change(const TDesC& msg)
{
	CALLSTACKITEM(_L("CPeriodicTransfer::status_change"));

	iCb->info(this, msg);
}
