 /*
* ============================================================================
*  Name     : CContextbookContainer from ContextbookContainer.h
*  Part of  : contextbook
*  Created  : 14.7.2003 by 
*  Implementation notes:
*     Initial content was generated by Series 60 AppWizard.
*  Version  :
*  Copyright: 
* ============================================================================
*/

// INCLUDE FILES
#include "ContextbookContainer.h"

#include <eiklabel.h>  // for example label control
#include <bautils.h>
#include <sendui.h>
#include <smut.h>
#include <SenduiMtmUids.h>
#include <txtrich.h>
#include <akntitle.h> 
#include <aknnavi.h> 
 
#include "doublelinebox.h"
#include <avkon.mbg>
#include <contextbook.mbg>

#include <gulbordr.h>
#include <gulicon.h>

#include <flogger.h>

#include "file_output_base.h"


enum KEYCODES {
	JOY_LEFT = 0xF807,
	JOY_RIGHT = 0xF808,
	JOY_UP = 0xF809,
	JOY_DOWN = 0xF80A,
	JOY_CLICK = 0xF845,
	KEY_CALL = 0xF862,
	KEY_CANCEL = 0xF863
};

enum SCANCODES {
	CALL_SCAN = 0xC4,
	CANCEL_SCAN = 0xC5
};



// ================= MEMBER FUNCTIONS =======================

// ---------------------------------------------------------
// CContextbookContainer::ConstructL(const TRect& aRect)
// EPOC two phased constructor
// ---------------------------------------------------------
//
void CContextbookContainer::ConstructL(const TRect& aRect, phonebook_i* i_book, bool i_searchable, const TDesC& title, Cfile_output_base * aLog)
{
	iCurrentContactId=KErrNotFound;

	iLog = aLog;
	CreateWindowL();

	CEikStatusPane* sp=iEikonEnv->AppUiFactory()->StatusPane();
	CAknTitlePane* tp=(CAknTitlePane*)sp->ControlL(TUid::Uid(EEikStatusPaneUidTitle));

	HBufC* t=HBufC::NewL(title.Length());
	*t=title;
	tp->SetText(t);

	iLog->write_time();
	iLog->write_to_output(_L("Showing "));
	iLog->write_to_output(title);
	iLog->write_nl();

	sendui=CSendAppUi::NewL(0);

	searchable=i_searchable;

	resource_files=new (ELeave) CArrayFixFlat<TInt>(5);

	// for phonebook dialogs
	TFileName resfile=_L("z:\\System\\data\\PBKVIEW.rSC");
	BaflUtils::NearestLanguageFile(iEikonEnv->FsSession(), resfile); //for localization
	resource_files->AppendL(iEikonEnv->AddResourceFileL(resfile)); 
	
	book=i_book;
	book->set_observer(this);

	listbox=new (ELeave) doublelinebox(book,iLog);
	listbox->SetContainerWindowL(*this);
	listbox->ConstructL(this, EAknListBoxSelectionList);
	listbox->SetItemHeightL(40);
	listbox->View()->SetMatcherCursor(EFalse);

	CArrayPtr<CGulIcon> * icons = new (ELeave) CArrayPtrFlat<CGulIcon>(30);

	SetArrayIcons(icons, book->get_iconlist());

	listbox->ItemDrawer()->FormattedCellData()->SetIconArray(icons);

	listbox->SetListBoxObserver(this);
	listbox->Model()->SetItemTextArray(book->get_array());
	
	listbox->Model()->SetOwnershipType(ELbmDoesNotOwnItemArray);
	listbox->CreateScrollBarFrameL(ETrue);
	listbox->ScrollBarFrame()->SetScrollBarVisibilityL( CEikScrollBarFrame::EOff, CEikScrollBarFrame::EAuto);
	
	
	if (is_searchable()) 
	{
		edit=new (ELeave) CEikEdwin;
		edit->SetBorder(TGulBorder::ESingleBlack);
		edit->SetContainerWindowL(*this);
		edit->ConstructL();
		edit->AddEdwinObserverL(this);
		edit->SetFocus(ETrue);
	}

	phone=new (ELeave) phonehelper;
	phone->ConstructL();

	globalNote=CAknGlobalNote::NewL();

	pbkengine=CPbkContactEngine::Static();
	if (pbkengine) {
		owns_engine=false;
	} else {
		pbkengine=CPbkContactEngine::NewL();
		owns_engine=true;
	}

	SetRect(aRect);

    
	ActivateL();
}

// Destructor
CContextbookContainer::~CContextbookContainer()
{
	if (book) book->set_observer(0);

	for (int i=0; i<resource_files->Count(); i++) {
		iEikonEnv->DeleteResourceFile((*resource_files)[i]);
	}
	delete resource_files;
	delete listbox;
	delete edit;
	delete phone;
	delete sendui;
	delete globalNote;

	//icons->Reset();
	//delete icons;
		
	if (owns_engine) delete pbkengine;
}

// ---------------------------------------------------------
// CContextbookContainer::SizeChanged()
// Called by framework when the view size is changed
// ---------------------------------------------------------
//
void CContextbookContainer::SizeChanged()
{
	TRect lb_rect=Rect();
	if (listbox) {

		if (edit) {
			lb_rect.Resize(-8, -20);
		} else {
			lb_rect.Resize(-8, 0);
		}

		lb_rect.Move(4, 0);
                listbox->SetRect(lb_rect);
	}

	if (edit) {
		TRect edit_rect=Rect();
		edit_rect.SetHeight(19);
		edit_rect.Resize(-8, 0);
		edit_rect.Move(4, lb_rect.Height());
		edit->SetRect(edit_rect);
		edit->SetFocus(ETrue);
	}
}

// ---------------------------------------------------------
// CContextbookContainer::CountComponentControls() const
// ---------------------------------------------------------
//
TInt CContextbookContainer::CountComponentControls() const
{
	if (edit) {
		return 2;
	} else {
		return 1;
	}
}

// ---------------------------------------------------------
// CContextbookContainer::ComponentControl(TInt aIndex) const
// ---------------------------------------------------------
//
CCoeControl* CContextbookContainer::ComponentControl(TInt aIndex) const
{
	switch ( aIndex )
        {
        case 0:
		return listbox;
	case 1:
		return edit;
        default:
		return NULL;
        }
}

// ---------------------------------------------------------
// CContextbookContainer::Draw(const TRect& aRect) const
// ---------------------------------------------------------
//
void CContextbookContainer::Draw(const TRect& aRect) const
{
	CWindowGc& gc = SystemGc();
	// TODO: Add your drawing code here
	// example code...
	gc.SetPenStyle(CGraphicsContext::ENullPen);

	gc.SetBrushColor(KRgbWhite);
	gc.SetBrushStyle(CGraphicsContext::ESolidBrush);
	gc.DrawRect(aRect);

	if (edit) {
		TGulBorder border(TGulBorder::ESingleBlack);
		gc.SetPenStyle(CGraphicsContext::ESolidPen);
		gc.SetBrushStyle(CGraphicsContext::ENullBrush);
		gc.SetPenColor(KRgbBlack);
		TRect edit_rect=Rect();
		edit_rect.SetHeight(19);
		edit_rect.Resize(-8, 0);
		edit_rect.Move(4, Rect().Height()-20);
		border.Draw(gc, edit_rect);
	}
}

// ---------------------------------------------------------
// CContextbookContainer::HandleControlEventL(
//     CCoeControl* aControl,TCoeEvent aEventType)
// ---------------------------------------------------------
//
void CContextbookContainer::HandleControlEventL(
						CCoeControl* /*aControl*/,TCoeEvent /*aEventType*/)
{
	// TODO: Add your control event handler code here
}


TKeyResponse CContextbookContainer::OfferKeyEventL(const TKeyEvent &aKeyEvent, TEventCode aType)
{
		if(listbox && (aKeyEvent.iCode==JOY_UP || aKeyEvent.iCode==JOY_DOWN || aKeyEvent.iCode==JOY_CLICK))
		{
			return listbox->OfferKeyEventL(aKeyEvent,aType);
		}	
		else if(aKeyEvent.iCode==KEY_CALL) 
		{
			call_current();
			return EKeyWasConsumed;
		} 
		else if(edit) 
		{
			TKeyResponse ret;
			ret=edit->OfferKeyEventL(aKeyEvent, aType);
			filter();
			return ret;
		} 
		else return EKeyWasNotConsumed;
}

void CContextbookContainer::before_change()
{
	if (! listbox ) {
		// still constructing
		return;
	}
	TInt idx=get_current_idx();
	TInt id=book->GetContactId(idx);
	if (id!=KErrNotFound) iCurrentContactId=id;
}

void CContextbookContainer::contents_changed()
{
	if (!listbox ) {
		// still constructing
		return;
	}

	// in case new icons were added
	SetArrayIcons(listbox->ItemDrawer()->FormattedCellData()->IconArray(), book->get_iconlist());
	
	listbox->HandleItemRemovalL();
	TInt idx=book->GetIndex(iCurrentContactId);
	if (idx==KErrNotFound) 
	{
		listbox->SetCurrentItemIndexAndDraw(0);
	} 
	else 
	{
		listbox->SetCurrentItemIndexAndDraw(idx);
	}
	DrawNow();	
}

void CContextbookContainer::exiting()
{
	book=0;
}

TInt CContextbookContainer::get_current_idx()
{
	return listbox->CurrentItemIndex();
}

void CContextbookContainer::sms_current()
{
	TPtrC no( book->get_phone_no(listbox->CurrentItemIndex()) );

	if (no!=KNullDesC) {
		iLog->write_time();
		iLog->write_to_output(_L("Smsing: "));
		iLog->write_to_output(listbox->Model()->ItemTextArray()->MdcaPoint(listbox->View()->CurrentItemIndex()));
		iLog->write_nl();
		CParaFormatLayer* paraf=CParaFormatLayer::NewL();
		CleanupStack::PushL(paraf);

		CCharFormatLayer* charf=CCharFormatLayer::NewL();
		CleanupStack::PushL(charf);

		CRichText* body;
		body=CRichText::NewL(paraf, charf);
		CleanupStack::PushL(body);
		
		CDesCArrayFlat* a=new CDesCArrayFlat(1);
		CleanupStack::PushL(a);
		a->AppendL(no);
		//s->CreateAndSendMessageL(KSenduiMtmSmsUid, 0, 0, KNullUid, a, 0, EFalse);
		sendui->CreateAndSendMessageL(KUidMsgTypeSMS, body, 0, KNullUid, a);
		CleanupStack::PopAndDestroy(4); //a, body, paraf, charf
	}
}

void CContextbookContainer::call_current()
{
	TInt current_index = listbox->CurrentItemIndex();
	if (current_index < 0) return;
	TPtrC no( book->get_phone_no(listbox->CurrentItemIndex()) );

	if (no!=KNullDesC) {
		iLog->write_time();
		iLog->write_to_output(_L("Calling: "));
		iLog->write_to_output(listbox->Model()->ItemTextArray()->MdcaPoint(listbox->View()->CurrentItemIndex()));
		iLog->write_nl();
		TInt ret=phone->make_call(no);
		/*if (ret!=KErrNone) {
			TBuf<50> msg;
			msg.Format(_L("Error %d"), ret);
			globalNote->ShowNoteL(EAknGlobalInformationNote, msg);
		}*/
	}
}

void CContextbookContainer::HandleListBoxEventL(CEikListBox* aListBox,TListBoxEvent aEventType)
{
	if(aListBox == listbox)
	{
		switch(aEventType)
		{
		case EEventEnterKeyPressed:
			call_current();
			break;
		}
	}
}

void CContextbookContainer::HandleEdwinEventL(CEikEdwin* aEdwin,TEdwinEvent aEventType)
{
	
}

void CContextbookContainer::filter()
{
	if (edit) {
		TBuf<20> buf;
		edit->GetText(buf);
		TBool changes = book->filter(buf);
		if (changes)
		{
			//((doublelinebox*)listbox)->LogVisibleItems(listbox->View()->CurrentItemIndex());
		}
	}
	RDebug::Print(_L("filtr"));
	
}

bool CContextbookContainer::is_searchable() const
{
	return searchable;
}

void  CContextbookContainer::SetArrayIcons(CArrayPtr<CGulIcon> * icons, TIconList * list)
{
	icons->ResetAndDestroy();

	//0 - empty Icon
	icons->AppendL(iEikonEnv->CreateIconL(_L("Z:\\system\\data\\avkon.mbm"), 
			EMbmAvkonQgn_stat_bt_blank, 
			EMbmAvkonQgn_stat_bt_blank +1));
	
	TBuf<128> KMbmFile;
#ifndef __WINS__
	KMbmFile=_L("c:\\system\\apps\\CONTEXTBOOK\\contextbook.mbm");
#else
	KMbmFile=_L("z:\\system\\apps\\CONTEXTBOOK\\contextbook.mbm");
#endif
	//1 - speaker ON
	icons->AppendL(iEikonEnv->CreateIconL(KMbmFile, 
			EMbmContextbookSpeaker_on,
			EMbmContextbookSpeaker_on_mask));

	//2 - speaker OFF
	icons->AppendL(iEikonEnv->CreateIconL(KMbmFile, 
			EMbmContextbookSpeaker_off, 
			EMbmContextbookSpeaker_off_mask));

	//3 - speaker ON grey
	icons->AppendL(iEikonEnv->CreateIconL(KMbmFile, 
			EMbmContextbookSpeaker_on_grey, 
			EMbmContextbookSpeaker_on_grey_mask));

	//4 - speaker OFF grey
	icons->AppendL(iEikonEnv->CreateIconL(KMbmFile, 
			EMbmContextbookSpeaker_off_grey, 
			EMbmContextbookSpeaker_off_grey_mask));

	//5 - user active
	icons->AppendL(iEikonEnv->CreateIconL(KMbmFile, 
			EMbmContextbookUser_active, 
			EMbmContextbookUser_active_mask));

	//6 - user inactive 
	icons->AppendL(iEikonEnv->CreateIconL(KMbmFile, 
			EMbmContextbookUser_inactive, 
			EMbmContextbookUser_inactive_mask));
	
	//7 - vibrator ON
	icons->AppendL(iEikonEnv->CreateIconL(KMbmFile, 
			EMbmContextbookVibrator_on, 
			EMbmContextbookVibrator_on_mask));
	
	//8 - vibrator OFF 
	icons->AppendL(iEikonEnv->CreateIconL(KMbmFile, 
			EMbmContextbookVibrator_off,
			EMbmContextbookVibrator_off_mask));

	//9 - vibrator ON grey
	icons->AppendL(iEikonEnv->CreateIconL(KMbmFile, 
			EMbmContextbookVibrator_on_grey,
			EMbmContextbookVibrator_on_grey_mask));

	//10 - vibrator OFF grey
	icons->AppendL(iEikonEnv->CreateIconL(KMbmFile, 
			EMbmContextbookVibrator_off_grey,
			EMbmContextbookVibrator_off_grey_mask));

	//11 - user_inactive level 1
	icons->AppendL(iEikonEnv->CreateIconL(KMbmFile, 
			EMbmContextbookUser_inactive_lvl_1,
			EMbmContextbookUser_inactive_mask));

	//12 - user_inactive level 2
	icons->AppendL(iEikonEnv->CreateIconL(KMbmFile, 
			EMbmContextbookUser_inactive_lvl_2,
			EMbmContextbookUser_inactive_mask));

	//13 - user_inactive level 3
	icons->AppendL(iEikonEnv->CreateIconL(KMbmFile, 
			EMbmContextbookUser_inactive_lvl_3,
			EMbmContextbookUser_inactive_mask));
	
	for (int i=0;i<list->Count();i++)
	{
		TInt iconId = (*list)[i];
		TInt maskId = iconId+1;
		icons->AppendL(iEikonEnv->CreateIconL(_L("Z:\\system\\data\\avkon.mbm"), iconId, maskId));
	}
}
