#include "call_log.h"
#include <eikenv.h>

#include <avkon.mbg>
#include <cpbkcontactitem.h>

#include <pbkiconinfo.h> 

#include <rpbkviewresourcefile.h>

#include "contextbookcontainer.h"


call_log::call_log() : CCheckedActive(EPriorityIdle, _L("call_log")), current_state(IDLE)
{
}

call_log::~call_log()
{
	if (obs) obs->exiting();

	Cancel();
	delete recent;
	delete logclient;
	delete logfilter;
	delete current_array;
	delete current_nos;
	delete all_array;
	delete all_nos;
	iconList->Close();
	delete iconList;
	delete iContactToItems;
}

void call_log::ConstructL()
{
	iContactToItems=CGenericIntMap::NewL();

	current_array = new (ELeave) CPtrCArray(8);
	current_nos = new (ELeave) CPtrCArray(8);

	all_array = new (ELeave) CDesCArrayFlat(50);
	all_nos = new (ELeave) CDesCArrayFlat(50);

	iconList = new (ELeave) TIconList();
	iconList->Append(EMbmAvkonQgn_stat_bt_blank);

	eng=CPbkContactEngine::Static();

	if (eng) {
		owns_engine=false;
	} else {
		eng=CPbkContactEngine::NewL();
		owns_engine=true;
	}

	logclient=CLogClient::NewL(CEikonEnv::Static()->FsSession());
	recent=CLogViewRecent::NewL(*logclient);
	logfilter=CLogFilter::NewL();
	logfilter->SetDirection(_L("Outgoing"));
	logfilter->SetEventType(KLogCallEventTypeUid);

	CActiveScheduler::Add(this);
	do_refresh=false; new_from=0;
	refresh();
}

void call_log::refresh()
{
	if (current_state!=IDLE) 
	{
		do_refresh=true;
		return;
	}

	getting_first=true;
	
	new_from=all_array->Count();
	
	TBool issued = recent->SetRecentListL(KLogNullRecentList, *logfilter, iStatus);
	
	if (issued && iStatus==KRequestPending) {
		current_state=WAITING_SET;
		SetActive();
	} else {
		current_state=IDLE;
	}
}

void call_log::ReRead()
{
	refresh();
}

MDesCArray* call_log::get_array()
{
	return current_array;
}

TIconList * call_log::get_iconlist()
{
	return iconList;
}

bool call_log::filter(const TDesC& /*substr*/, bool /*force*/)
{
	return false;
}

TPtrC call_log::get_phone_no(TInt index)
{
	return (*current_nos)[index];
}

bool call_log::handle_event(const CLogEvent& ev)
{
	if (ev.Id()==last_seen_id ) return false;
	

	if (getting_first) 
	{
		
		last_seen_id=ev.Id();
		getting_first=false;
	}

	TTime tt=ev.Time();

	TLocale locale;
	TTimeIntervalSeconds offset(locale.UniversalTimeOffset());
	tt+=offset;
	if (locale.QueryHomeHasDaylightSavingOn()) {
		TTimeIntervalHours ds(1);
		tt+=ds;
	}

	all_nos->AppendL(ev.Number());

	TDateTime t=tt.DateTime();

	TBuf<200> text;
	text.Format(_L("%02d.%02d %02d:%02d "),
		(TInt)t.Day()+1, (TInt)t.Month()+1,
		(TInt)t.Hour(), (TInt)t.Minute() );
	if (ev.RemoteParty().Length()>0) {
		text.Append(ev.RemoteParty());
	} else {
		text.Append(ev.Number());
	}
	text.Append(_L("\t"));
	
	// Converting phone number to Int
	TInt nbDigits = 0;
	TInt phoneno = eng->Database().TextToPhoneMatchNumber(ev.Number(), nbDigits,8);
	
	
	// Finding Contacts fitting the number...
	CContactIdArray * current_contact_ids = 0;
	TRAPD(errorCode, current_contact_ids = eng->Database().MatchPhoneNumberL(phoneno) );
	
	// getting icon
	TInt iconId = -1;
	TInt index=0;
	
	if (errorCode != KErrNotFound)
	{
		TPbkContactItemField * f = (eng->ReadContactL((*current_contact_ids)[0]))->FindNextFieldWithPhoneNumber(ev.Number(),8,index);
	
	
		if (f) {
			CCoeEnv *env = CEikonEnv::Static();
			RPbkViewResourceFile pbkRes( *env);
			pbkRes.OpenL();
			
			CPbkIconInfoContainer * container = CPbkIconInfoContainer::NewL();  
			CleanupStack::PushL(container);
		
			iconId = container->Find(f->IconId())->IconId();
			CleanupStack::PopAndDestroy(); //container
			pbkRes.Close();
		}
	
		if (iconId != -1) //icon was found
		{
			//check if icon exists already in list
			TInt index = iconList->Find(iconId);

			if (index != KErrNotFound)
			{
				TBuf<5> icon_int;
				icon_int.Format(_L("\t%d"),index + KIconOffset);
				text.Append(icon_int);
			}
			else
			{
				iconList->Append(iconId);
				index = iconList->Find(iconId);
				TBuf<5> icon_int;
				icon_int.Format(_L("\t%d"),index + KIconOffset);
				text.Append(icon_int);
			}
			
		}
		else // no icon found -> use empty icon
		{
			text.Append(_L("\t0"));
		}
	}
	else
	{
		text.Append(_L("\t0"));
	}
	text.Append(_L("\t0\t0"));
	
	all_array->AppendL(text);
	RDebug::Print(text);
	return true;
}

void call_log::copy_new_to_current()
{
	// trying to be efficient
	int new_count;

	new_count=all_array->Count()-(new_from);
	
	if (new_count <= 0) return;

	int i;
    current_array->ResizeL(new_count);
	current_nos->ResizeL(new_count);

    for (i=0;i<new_count;i++)
	{
		
		current_array->At(i).Set((*all_array)[new_from+i]);
		current_nos->At(i).Set((*all_nos)[new_from+i]);	
	}
}
		

void call_log::CheckedRunL()
{
	if (iStatus!=KErrNone) {
		Cancel();
		//TODO: err msg
		return;
	}
	
	switch (current_state) {
	
	case WAITING_SET:

		if (recent->FirstL(iStatus) && iStatus==KRequestPending) 
		{
			
			SetActive();
			current_state=WAITING_NEXT;
		} 
		else 
		{
			current_state=IDLE;
		}
		break;

	case WAITING_NEXT:
		
		if ( handle_event(recent->Event()) )
		{
			if (recent->NextL(iStatus) && iStatus==KRequestPending) 
			{
				SetActive();
			} 
			else	
			{
				copy_new_to_current();
				
				if (obs) obs->contents_changed();
				
				current_state=IDLE;
			}
		} 
		else 
		{
			// seen rest already
			copy_new_to_current();
			if (obs) obs->contents_changed();
			current_state=IDLE;
		}
		break;
	}

	

	if (current_state==IDLE && do_refresh) {
		// redo queued
		do_refresh=false;
		refresh();
	}
}

void call_log::DoCancel()
{

	if (current_state!=IDLE && recent) recent->Cancel();
	current_state=IDLE;
}

void call_log::set_observer(phonebook_observer* i_obs)
{
	obs=i_obs;
}
