#include "phonebook.h"

#include <cpbkcontactengine.h> 
#include <cpbkcontactiter.h> 
#include <cpbkcontactitem.h> 

#include <bamatch.h>
#include <cntdb.h>
#include <CPbkPhoneNumberSelect.h>
#include "presence_data.h"
#include <e32math.h>
#include <avkon.mbg>

#include <contextbook.rsg>
#include "contextbook.hrh"

#include <eikenv.h>

#pragma warning(disable: 4706)



contact* contact::NewL(TContactItemId i_id, const TDesC& i_first_name, const TDesC& i_last_name) {
		contact* c=new (ELeave) contact;
		CleanupStack::PushL(c);
		c->id=i_id;
		
		c->first_name=HBufC::NewL(i_first_name.Length());
		*(c->first_name)=i_first_name;

		c->last_name=HBufC::NewL(i_last_name.Length());
		*(c->last_name)=i_last_name;

		c->name=HBufC::NewL(i_first_name.Length()+i_last_name.Length()+50);
		*(c->name)=i_last_name;
		if (i_last_name.Length()>0 && i_first_name.Length()>0) {
			c->name->Des().Append(_L(" "));
		}
		c->name->Des().Append(i_first_name);
		c->name->Des().Append(_L("\t \t00\t00\t00")); //type No_icon_to_display = 0

		CleanupStack::Pop();

		return c;
	}

TBool contact::IsPresenceOutOfDate()
	{
		TBool old = EFalse;
		const TInt KFreshnessInterval = 15; // in minutes
		if (this->presence != NULL)
		{
			TTime now = TTime();
			now.HomeTime();
		#ifdef __WINS__
			now+=TTimeIntervalHours(2);
		#endif
			TTimeIntervalMinutes m;
			now.MinutesFrom(this->presence->SendTimeStamp(), m);
			if (m.Int() > KFreshnessInterval)
			{
				old = ETrue;
				this->set_presence(this->presence); // needed for the change of icons ...
			}
		}
		return old;
	}

	

	
void contact::set_presence(const CPresenceData* data)
{
		const TInt KFreshnessInterval = 15; // in minutes
			
		if (!data || !data->LocInfo().iCurrentOrPrevious) return;

		presence=data;
		
		TBool out_of_date = EFalse;
		TTime now = TTime();
		now.HomeTime();
#ifdef __WINS__
		now+=TTimeIntervalHours(2);
#endif
		TTimeIntervalMinutes m;
		now.MinutesFrom(presence->SendTimeStamp(), m);
		if (m.Int() > KFreshnessInterval)
		{
			out_of_date = ETrue;	
		}
	
		TInt len=name->Length()+data->LocInfo().iCurrentOrPrevious->Length()+120;
		name->Des().Zero();
		while (len>name->Des().MaxLength()) {
			name=name->ReAllocL(name->Des().MaxLength()*2);
		}

		name->Des().Append(*last_name);
		if (last_name->Length()>0 && first_name->Length()>0) {
			name->Des().Append(_L(" "));
		}
		name->Des().Append(*first_name);
		name->Des().Append(_L("\t"));
		
	
		// - Location Info -----------------------------------------------------

		if ( presence->LocInfo().iFlags & TLocInfo::EPrevious ) {
			//name->Des().Append(_L("prev: "));
			TBuf<6> prev;

			CEikonEnv::Static()->ReadResourceAsDes16(prev, R_PREVIOUS_CAPTION);
			
			name->Des().Append(prev);
			name->Des().Append(_L(" "));
			
		}

 		name->Des().Append(*(presence->LocInfo().iCurrentOrPrevious));
		
		TTime stamp = presence->LocInfo().iCoPUpdated;

		TTimeIntervalDays days = now.DaysFrom(stamp);
		if (days.Int() > 2)
		{
			name->Des().Append(_L(" (+2D)"));
		}
		else if (days.Int() >= 1)
		{
			name->Des().Append(_L(" (+1D)"));
		}
		else
		{
			TTimeIntervalHours hours;
			now.HoursFrom(stamp, hours);
			TTimeIntervalMinutes minutes;
			now.MinutesFrom(stamp, minutes);

			TBuf<10> s;
			s.Format(_L(" (%dh"), hours.Int());
			name->Des().Append(s);

			TInt min = (minutes.Int() - (hours.Int() * 60));
			
			if (min < 15)
			{
				name->Des().Append(_L("00)"));
			}
			else if (15 <= min && min < 30)
			{
				name->Des().Append(_L("15)"));
			}
			else if (30 <= min && min <45 )
			{
				name->Des().Append(_L("30)"));
			}
			else if (45 <= min )
			{
				name->Des().Append(_L("45)"));
			}
		}

		// - User activity --------------------------------------------------
		if (presence->ActivityInfo().iMode == TActivityInfo::EActive )
		{
			if (out_of_date) 
			{
				// grey hand
				name->Des().Append(_L("\t6"));
			}
			else
			{
				// color hand
				name->Des().Append(_L("\t5"));
			}
		}
		else if (presence->ActivityInfo().iMode == TActivityInfo::EIdle)
		{
			TTimeIntervalMinutes minutes;
			now.MinutesFrom(presence->ActivityInfo().iUpdated, minutes);

			if (minutes.Int() <= 10)
			{
				//hand icon grey level 1
				name->Des().Append(_L("\t11"));
			}
			else if (minutes.Int() <= 60)
			{
				//hand icon grey level 2
				name->Des().Append(_L("\t12"));
			}
			else if (minutes.Int() <= 240)
			{
				//hand icon grey level 2
				name->Des().Append(_L("\t13"));
			}
			else
			{
				// no icon
				name->Des().Append(_L("\t00"));
			}
		}
		else // unknown
		{
			// no icon
			name->Des().Append(_L("\t00"));
		}
		
			
		// - Speaker Info -----------------------------------------------------
		if ( presence->ProfileInfo().iRingingType == 4 )
		{
			if (out_of_date) 
			{
				// speaker off grey
				name->Des().Append(_L("\t04"));		
			}
			else
			{
				// speaker off color
				name->Des().Append(_L("\t02"));
			}
			
		}
		else
		{
			if (out_of_date) 
			{
				// speaker on grey
				name->Des().Append(_L("\t03"));		
			}
			else
			{
				// speaker on
				name->Des().Append(_L("\t01"));
		
			}
		}

		// - Vibrator Info -----------------------------------------------------
		if ( presence->ProfileInfo().iVibra )
		{
			if (out_of_date) 
			{
				// vibra on grey
				name->Des().Append(_L("\t09"));		
			}
			else
			{
				// vibra on 
				name->Des().Append(_L("\t07"));
		
			}
		}
		else
		{
			if (out_of_date) 
			{
				// vibra off grey
				name->Des().Append(_L("\t10"));		
			}
			else
			{
				//vibra off
				name->Des().Append(_L("\t08"));
		
			}
		}
		RDebug::Print(*name);

	}

	contact::~contact() {
		delete name;
		delete first_name;
		delete last_name;
	}


class contacts_key : public TKey {
public:
	contacts_key(CArrayFixFlat<contact*> & i_arr) : arr(i_arr) { }
	TInt Compare(TInt aLeft,TInt aRight) const {
		contact* left=arr[aLeft];
		contact* right=arr[aRight];
		if (left->has_nick && !right->has_nick) return -1;
		if (right->has_nick && !left->has_nick) return 1;
		return left->name->Des().Compare(right->name->Des());
	}
private:
	CArrayFixFlat<contact*>& arr;
};

class contacts_swap : public TSwap {
public:
	contacts_swap(CArrayFixFlat<contact*> & i_arr) : arr(i_arr) { }
	void Swap(TInt aLeft,TInt aRight) const {
		contact* tmp=arr[aLeft];
		arr[aLeft]=arr[aRight];
		arr[aRight]=tmp;
	}
private:
	CArrayFixFlat<contact*>& arr;
};

phonebook::phonebook(MApp_context& Context, CJabberData& JabberData, CPresenceHolder& PresenceHolder) : 
	MContextBase(Context), iJabberData(JabberData), iPresenceHolder(PresenceHolder)
{
}

phonebook::~phonebook()
{
	if (obs) obs->exiting();

	delete current_array;
	delete current_ids;
	
	if (contacts) {
		reset_contacts();
	}
	delete contacts;

	delete contactitem;
	delete chnot;

	if (owns_engine) {
		delete eng;
	}

	delete iContactToIndex;

	iconList->Close();
	delete iconList;
}

TInt phonebook::GetContactId(TInt Index)
{
	if (Index<0 || Index >= current_ids->Count()) return KErrNotFound;
	return (*current_ids)[Index];
}

void phonebook::ConstructL()
{
	iContactToIndex=CGenericIntMap::NewL();

	current_array = new (ELeave) CPtrCArray(50);
	current_ids=new (ELeave) CArrayFixFlat<TContactItemId>(50);
		
	eng=CPbkContactEngine::Static();

	if (eng) {
		owns_engine=false;
	} else {
		eng=CPbkContactEngine::NewL();
		owns_engine=true;
	}
	
	read_db();

	chnot=eng->CreateContactChangeNotifierL(this);

	iconList = new (ELeave) TIconList;
	iconList->Append(EMbmAvkonQgn_stat_bt_blank);
}

void phonebook::read_db()
{
	read_data(eng->CreateContactIteratorLC(ETrue));
	CleanupStack::PopAndDestroy();
	User::QuickSort(contacts->Count(), contacts_key(*contacts), contacts_swap(*contacts));
	
	filter(previous_filter, true);
}

void phonebook::HandleDatabaseEventL(TContactDbObserverEvent /*aEvent*/)
{
	read_db();
}

void phonebook::PresenceChangedL(TInt ContactId, const CPresenceData& Info)
{
	if (obs) obs->before_change();

	TInt idx=(TInt)iContactToIndex->GetData(ContactId);
	if (!idx) return;
	contact* c=(*contacts)[idx-1];
	c->set_presence(&Info);
	TInt curr_idx=c->current_idx;
	if (!curr_idx) return;
	(*current_array)[curr_idx-1].Set(c->name->Des());

	if (obs) obs->contents_changed();
}

void phonebook::Notify(const TDesC & aMessage)
{
	// no impl
}

phonebook_observer * phonebook::get_observer()
{
	return obs;
}

TInt phonebook::GetIndex(TInt ContactId)
{
	TInt idx=(TInt)iContactToIndex->GetData(ContactId);
	if (!idx) return KErrNone;
	contact* c=(*contacts)[idx-1];
	TInt curr_idx=c->current_idx;
	if (!curr_idx) return KErrNone;
	return curr_idx-1;
}

void phonebook::reset_contacts()
{
	//iContactToIndex->Reset();

	for (int i=0; i<contacts->Count(); i++) {
		delete (*contacts)[i];
	}
	contacts->Reset();
}

void phonebook::read_data(CPbkContactIter * iter)
{
	if (contacts) {
		TInt c=contacts->Count();
		reset_contacts();
		contacts->SetReserveL(c+1);
	} else {
		contacts=new (ELeave) CArrayFixFlat<contact*>(50);
	}

	CPbkContactItem *item=0;
	TBuf<100> first_name;
	TBuf<100> last_name;
	TBuf<100> jabber_nick;
	
	for (iter->FirstL(); item=iter->CurrentL(); iter->NextL()) {
		
		TPbkContactItemField* f;
		
		f=item->FindField(EPbkFieldIdLastName);
		if (f) {
			last_name=f->Text();
		} else {
			last_name=_L("");
		}
		f=item->FindField(EPbkFieldIdFirstName);
		if (f) {
			first_name=f->Text();
		} else {
			first_name=_L("");
		}
		contact* c=contact::NewL(item->Id(), first_name, last_name);
		c->set_presence(iPresenceHolder.GetPresence(item->Id()));
		if (iJabberData.GetJabberNickL(item->Id(), jabber_nick)) {
			c->has_nick=true;
		}
		contacts->AppendL(c);
		
	}
}

void phonebook::ReRead()
{
	read_db();
}

MDesCArray* phonebook::get_array()
{
	return current_array;
}

TIconList * phonebook::get_iconlist()
{	
	/*TIconList * aDummyList = new (ELeave) TIconList;
	aDummyList->Append(EMbmAvkonQgn_stat_bt_blank);
	return aDummyList;
*/
	return iconList;
}

TPtrC phonebook::get_phone_no(TInt index)
{
	delete contactitem;

	TContactItemId id=(*current_ids)[index];
	if (id!=0) {
		contactitem=eng->ReadContactL(id);
		CPbkPhoneNumberSelect* sel=new (ELeave) CPbkPhoneNumberSelect;
		return sel->ExecuteLD(*contactitem, NULL, EFalse);
	}
	return TPtrC();
}

void phonebook::set_observer(phonebook_observer* i_obs)
{
	obs=i_obs;
}

CArrayFixFlat<contact*>* phonebook::GetContacts()
{
	return contacts;
}

bool phonebook::filter(const TDesC& substr, bool force)
{
	if (!force && ! previous_filter.Compare(substr)) return false;
	
	if (obs) obs->before_change();

	current_ids->Reset();
	current_array->Reset();
	TBuf<100> name;
	
	RIncrMatcherBuf<30> matcher;
	matcher.SetMatchText(substr);
	
	bool match=false;
	contact* ci;
	TInt idx=0;
	for (int i=0; i<contacts->Count(); i++) {
		ci=(*contacts)[i];
		if (substr.Length()==0) match=true;
		else {
			if (matcher.IsMatchF( *(ci->first_name) ) || 
				matcher.IsMatchF( *(ci->last_name) ) ) match=true;
		}
		if (match) {
			current_array->AppendL(ci->name->Des() );
			current_ids->AppendL( ci->id);
			iContactToIndex->AddDataL(ci->id, (void*) (i+1), true);
			ci->current_idx=idx+1;
			idx++;
			match=false;
		} else {
			ci->current_idx=0;
			iContactToIndex->AddDataL(ci->id, (void*) (0), true);
		}
	}
	previous_filter=substr;

	if (obs) obs->contents_changed();
	return true;
}

CPbkContactEngine* phonebook::get_engine()
{
	return eng;
}


// -------------------------------------------------------------

const TInt CPresenceUpdater::KTimeOut = 60;

CPresenceUpdater * CPresenceUpdater::NewL(phonebook& book)
{
	auto_ptr<CPresenceUpdater> ret(new (ELeave) CPresenceUpdater(book));
	ret->ConstructL();
	return ret.release();
}

CPresenceUpdater::CPresenceUpdater(phonebook& book) : book(book), iWaiting(EFalse)
{
}

CPresenceUpdater::~CPresenceUpdater()
{
	delete iWait;
}

void CPresenceUpdater::ConstructL()
{
	
	iWait=CTimeOut::NewL(*this);
	
	Start();
}

void CPresenceUpdater::Start()
{
	if (!iWaiting)
	{
		iWait->Wait(KTimeOut);
		iWaiting= ETrue;
	}
}

void CPresenceUpdater::Stop()
{
	iWait->Reset();	
	iWaiting = EFalse;
}



/*
void CPresenceUpdater::CheckedRunL()
{
	switch(iStatus.Int())
	{
		case KErrNone:

		default:
		RDebug::Print(_L("here"));
		break;
	}
}
*/
void CPresenceUpdater::expired()
{
	RDebug::Print(_L("refreshing..."));
	CArrayFixFlat<contact*> * contacts = book.GetContacts();
	
	for (int i=0; i< contacts->Count();i++)
	{
		if ( (*contacts)[i]->presence )
		{
			book.PresenceChangedL( (*contacts)[i]->id, *((*contacts)[i]->presence) );
		}
	}
	
	if (!iWaiting)
	{
	
	}
	else
	{
		iWaiting = EFalse;
		Start();
	}
}
